(** Allows to store elements in a set, and execute
    a function on each element only once. 
    @author: Phillip Heidegger
*)


open ExtSSet
open ExtSSetImp

module type S = sig
  type 'a t
  type elt

  (** Creates a new flag instance *)
  val create : unit -> 'a t

  val clean : 'a t -> unit

  (** [visit set e f] executes [f] on [e], if
      [e] is not part of the set. Before [f] is
      executed, [e] is stored in the set to
      prevent executing [f] on the same element
      another time. [Some (f elm)] is returned
      in this case.

      If [e] is part of the set, [None] is returned
      and [f] is not executed.
  *)
  val visit : ?cmp:(elt -> elt -> bool) -> 'a t -> elt -> (elt -> 'a) -> 'a option


  (** Same as visit, but unit is returned. *)
  val visit_u : unit t -> elt -> (elt -> unit) -> unit

  (** Same as visit, but if the element is part of
      the set, e is returned as result of [visit_id set e f].
  *)
  val visit_id : elt t -> elt -> (elt -> elt) -> elt
end

module Make :  functor(Ord: OrderedType) -> S
  with type elt = Ord.t
